#ifndef MSP_GL_LIGHT_H_
#define MSP_GL_LIGHT_H_

#include <vector>
#include "color.h"
#include "vector.h"

namespace Msp {
namespace GL {

class Matrix;
class ProgramData;

/**
Stores properties of a single light source.

Lights do not cast shadows by themselves.  See ShadowMap for that.
*/
class Light
{
private:
	enum ParameterMask
	{
		DIFFUSE = 1,
		SPECULAR = 2,
		POSITION = 4,
		SPOT_DIR = 8,
		SPOT_EXP = 16,
		SPOT_CUTOFF = 32,
		ATTENUATION = 64
	};

	Color diffuse;
	Color specular;
	Vector4 position;
	Vector3 spot_dir;
	float spot_exp;
	float spot_cutoff;
	float attenuation[3];

public:
	Light();
	~Light();

private:
	void update_parameter(int, int = -1) const;

public:
	/** Sets the diffuse (direction-independent) color of the Light.  Provided
	to shaders with the name light_sources[i].diffuse. */
	void set_diffuse(const Color &c);

	/** Sets the specular (direction-dependent) color of the Light.  Provided to
	shaders with the name light_sources[i].diffuse. */
	void set_specular(const Color &c);

	const Color &get_diffuse() const { return diffuse; }
	const Color &get_specular() const { return specular; }

	/** Sets the position of the Light.  For a directional light, set the xyz
	components to a vector pointing towards the light and the w component to 0. */
	void set_position(const Vector4 &);

	const Vector4 &get_position() const { return position; }

	void set_spot_direction(const Vector3 &);
	void set_spot_exponent(float);
	void set_spot_cutoff(float);
	const Vector3 &get_spot_direction() const { return spot_dir; }
	float get_spot_exponent() const { return spot_exp; }
	float get_spot_cutoff() const { return spot_cutoff; }
	void set_attenuation(float, float, float);
	const float *get_attenuation() const { return attenuation; }

	/** Updates a ProgramData object with the uniforms for the Light.  A view
	matrix and light source index must be passed in. */
	void update_shader_data(ProgramData &, const Matrix &, unsigned) const;

	void bind() const { return bind_to(0); }
	void bind_to(unsigned) const;

	static const Light *current(unsigned = 0);
	static void unbind() { return unbind_from(0); }
	static void unbind_from(unsigned);
};

} // namespace GL
} // namespace Msp

#endif
